classdef class_guiStructure < handle
    properties
    % ACTIVE    
        % figure
        fig
        % axis    
        Ax
        % edite
        edit_inputFolder
        edit_elSize 
        edit_xAxScale
        edit_yAxScale
        edit_zAxScale
        edit_dispScale
        elementAdvice
        % slider
        slider_xAxScale
        slider_yAxScale
        slider_zAxScale
        % checkbox
        checkbox_uvwPlot
        checkbox_nodesPlot
        checkbox_nodesText
        checkbox_eleText
        % static text
        staticText_prePro
        staticText_postPro
        staticTxt_xAxScale1
        staticTxt_xAxScale2
        staticTxt_yAxScale1
        staticTxt_yAxScale2
        staticTxt_zAxScale1
        staticTxt_zAxScale2
        staticText_dispScale
        % radioGroup
        radioGroup
        otherRadios = [1 2 3];
        % button
        button_launch   
    % PASSIVE (portable variables)
        % vector of objects
        vecObj_nodes
        vecObj_elements
        vecObj_connections
        vecObj_forces
        vecObj_BCs
        % axis
        jointsXmin
        jointsXmax
        jointsZmin
        jointsZmax
        longerAxisLength
        shorterAxisLength
        % colors
        BCsColor        = [0.1686    0.7608    0.3686];
        nodesColor      = [0.3020    0.7451    0.9333];
        MquiverColor    = [1.0000    0.6000    0.7843];
        FquiverColor    = [0.0745    0.6863    0.7686];
        elementsColor   = [0.5647    0.5647    0.5647];
        % for help
        circleStep      = 0:pi/50:2*pi;
        onOffwarnConn
        onOffwarnDisp
        RSLTs
        qStart
        n
    end
    methods 
        function createStructure(obj)
            
            fce_welcomingPlot();
            
            obj.fig             = gcf; % current figure handle
            obj.fig.Renderer    = 'opengl';
            obj.fig.Name        = 'FEM Analysis of frame construction'; 
            obj.fig.NumberTitle = 'off'; 
            obj.fig.Position    = [0,0,1200,700]; % leftOffset lowerOffset width height
            
            obj.Ax                  = gca; % current axis handle
            obj.Ax.Position         = [0.23 0.08 0.73 0.9];
            obj.Ax.Color            = [1 1 1];
            obj.Ax.Box              = 'on';
            obj.Ax.FontSize         = 20;
            obj.Ax.XLabel.String    = 'Label - x';
            obj.Ax.XLabel.Color     = [0.0588    0.5255    0.7882];
            obj.Ax.YLabel.String    = 'Label - y';
            obj.Ax.YLabel.Color     = [0.4039    0.7216    0.0941];
            obj.Ax.ZLabel.String    = 'Label - z';
            obj.Ax.ZLabel.Color     = [0.9216    0.1137    0.4196];
            
            preStart = 0.94;
            preSpace = 0.06;
            
            postStart = 0.65;
            
        % PRE-PROCESSOR

            % object with properties of UIControl staticText_prePro
            obj.staticText_prePro                = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticText_prePro.Units           = 'normalized';
            obj.staticText_prePro.Position        = [0.008,preStart,0.15,0.05];
            obj.staticText_prePro.String          = 'PRE - Processor';
            obj.staticText_prePro.FontSize        = 20;
            
            % object with properties of UIControl edit_inputFolder
            obj.edit_inputFolder                    = uicontrol('Parent',obj.fig,'Style','edit');
            obj.edit_inputFolder.Units              = 'normalized';
            obj.edit_inputFolder.Position           = [0.008,preStart-0.04,0.15,0.05];
            obj.edit_inputFolder.String             = 'f o l d e r - n a m e';
            obj.edit_inputFolder.FontSize           = 15;
            
            % object with properties of UIControl elementAdvice
            obj.elementAdvice                   = uicontrol('Parent',obj.fig,'Style','edit');
            obj.elementAdvice.Units             = 'normalized';
            obj.elementAdvice.Position          = [0.008,preStart-0.04-preSpace,0.15,0.05];
            obj.elementAdvice.String            = '==( Only For Reading )==';
            obj.elementAdvice.FontSize          = 15;
            
            % object with properties of UIControl edit_elSize
            obj.edit_elSize                 = uicontrol('Parent',obj.fig,'Style','edit');
            obj.edit_elSize.Units           = 'normalized';
            obj.edit_elSize.Position        = [0.008,preStart-0.04-2*preSpace,0.15,0.05];
            obj.edit_elSize.String          = 'e l e m e n t - s i z e';
            obj.edit_elSize.FontSize        = 15;
            
            % object with properties of UIControl button_launch
            obj.button_launch                	= uicontrol('Parent',obj.fig,'Style','pushbutton');
            obj.button_launch.Units             = 'normalized';
            obj.button_launch.Position          = [0.008,preStart-0.04-3*preSpace,0.15,0.05];
            obj.button_launch.String            = 'L A U N C H';
            obj.button_launch.FontSize          = 20;
            obj.button_launch.BackgroundColor   = 'k';
            obj.button_launch.ForegroundColor   = 'white';
            
        % POST-PROCESSOR
        
            % object with properties of UIControl staticText_postPro
            obj.staticText_postPro                = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticText_postPro.Units           = 'normalized';
            obj.staticText_postPro.Position        = [0.008,postStart,0.15,0.05];
            obj.staticText_postPro.String          = 'POST - Processor';
            obj.staticText_postPro.FontSize        = 20;

            % object with properties of UIControl slider_xAxScale
            obj.slider_xAxScale            = uicontrol('Parent',obj.fig,'Style', 'slider');
            obj.slider_xAxScale.Units      = 'Normalized';
            obj.slider_xAxScale.Position   = [0.03,postStart-0.045,0.115,0.05];
            obj.slider_xAxScale.Min        = 0.3;
            obj.slider_xAxScale.Max        = 3;
            obj.slider_xAxScale.Value      = 1;
            obj.slider_xAxScale.SliderStep = [1/53 0.05];
            
            % object with properties of UIControl staticText_xAxScale1
            obj.staticTxt_xAxScale1                 = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticTxt_xAxScale1.Units           = 'normalized';
            obj.staticTxt_xAxScale1.Position        = [0.008,postStart-0.045,0.02,0.05];
            obj.staticTxt_xAxScale1.String          = '0.3';
            obj.staticTxt_xAxScale1.FontSize        = 15;
            
            % object with properties of UIControl staticText_xAxScale2
            obj.staticTxt_xAxScale2                 = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticTxt_xAxScale2.Units           = 'normalized';
            obj.staticTxt_xAxScale2.Position        = [0.145,postStart-0.045,0.02,0.05];
            obj.staticTxt_xAxScale2.String          = '3';
            obj.staticTxt_xAxScale2.FontSize        = 15;
            
            % object with properties of UIControl edit_xAxScale
            obj.edit_xAxScale                 = uicontrol('Parent',obj.fig,'Style','edit');
            obj.edit_xAxScale.Units           = 'normalized';
            obj.edit_xAxScale.Position        = [0.008,postStart-0.045-0.03,0.15,0.05];
            obj.edit_xAxScale.FontSize        = 15;
            obj.edit_xAxScale.String          = ['==( Only For Reading )=='];          
            
            % object with properties of UIControl slider_yAxScale
            obj.slider_yAxScale            = uicontrol('Parent',obj.fig,'Style', 'slider');
            obj.slider_yAxScale.Units      = 'Normalized';
            obj.slider_yAxScale.Position   = [0.03,postStart-0.15,0.115,0.05];
            obj.slider_yAxScale.Min        = 0.3;
            obj.slider_yAxScale.Max        = 3;
            obj.slider_yAxScale.Value      = 1;
            obj.slider_yAxScale.SliderStep = [1/53 0.05];
            obj.slider_yAxScale.Units      = 'Normalized';
            
            % object with properties of UIControl staticText_yAxScale1
            obj.staticTxt_yAxScale1                 = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticTxt_yAxScale1.Units           = 'normalized';
            obj.staticTxt_yAxScale1.Position        = [0.008,postStart-0.15,0.02,0.05];
            obj.staticTxt_yAxScale1.String          = '0.3';
            obj.staticTxt_yAxScale1.FontSize        = 15;
            
            % object with properties of UIControl staticText_yAxScale2
            obj.staticTxt_yAxScale2                 = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticTxt_yAxScale2.Units           = 'normalized';
            obj.staticTxt_yAxScale2.Position        = [0.145,postStart-0.15,0.02,0.05];
            obj.staticTxt_yAxScale2.String          = '3';
            obj.staticTxt_yAxScale2.FontSize        = 15;
            
            % object with properties of UIControl edit_yAxScale
            obj.edit_yAxScale                 = uicontrol('Parent',obj.fig,'Style','edit');
            obj.edit_yAxScale.Units           = 'normalized';
            obj.edit_yAxScale.Position        = [0.008,postStart-0.15-0.03,0.15,0.05];
            obj.edit_yAxScale.FontSize        = 15;
            obj.edit_yAxScale.String          = ['==( Only For Reading )=='];
            
            % object with properties of UIControl slider_zAxScale
            obj.slider_zAxScale            = uicontrol('Parent',obj.fig,'Style', 'slider');
            obj.slider_zAxScale.Units      = 'Normalized';
            obj.slider_zAxScale.Position   = [0.03,postStart-0.255,0.115,0.05];
            obj.slider_zAxScale.Min        = 0.3;
            obj.slider_zAxScale.Max        = 3;
            obj.slider_zAxScale.Value      = 1;
            obj.slider_zAxScale.SliderStep = [1/53 0.05];
            obj.slider_zAxScale.Units      = 'Normalized';
            
            % object with properties of UIControl staticText_zAxScale1
            obj.staticTxt_zAxScale1                 = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticTxt_zAxScale1.Units           = 'normalized';
            obj.staticTxt_zAxScale1.Position        = [0.008,postStart-0.255,0.02,0.05];
            obj.staticTxt_zAxScale1.String          = '0.3';
            obj.staticTxt_zAxScale1.FontSize        = 15;
            
            % object with properties of UIControl staticText_zAxScale2
            obj.staticTxt_zAxScale2                 = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticTxt_zAxScale2.Units           = 'normalized';
            obj.staticTxt_zAxScale2.Position        = [0.145,postStart-0.255,0.02,0.05];
            obj.staticTxt_zAxScale2.String          = '3';
            obj.staticTxt_zAxScale2.FontSize        = 15;
            
            % object with properties of UIControl edit_zAxScale
            obj.edit_zAxScale                 = uicontrol('Parent',obj.fig,'Style','edit');
            obj.edit_zAxScale.Units           = 'normalized';
            obj.edit_zAxScale.Position        = [0.008,postStart-0.255-0.03,0.15,0.05];
            obj.edit_zAxScale.FontSize        = 15;
            obj.edit_zAxScale.String          = ['==( Only For Reading )=='];
            
            % object with properties of UIControl staticText_dispScale
            obj.staticTxt_zAxScale2                 = uicontrol('Parent',obj.fig,'Style','text');
            obj.staticTxt_zAxScale2.Units           = 'normalized';
            obj.staticTxt_zAxScale2.Position        = [0.008,postStart-0.348,0.1,0.025];
            obj.staticTxt_zAxScale2.String          = 'd i s p - s c a l e:';
            obj.staticTxt_zAxScale2.FontSize        = 15;
            
            % object with properties of UIControl edit_dispScale
            obj.edit_dispScale                 = uicontrol('Parent',obj.fig,'Style','edit');
            obj.edit_dispScale.Units           = 'normalized';
            obj.edit_dispScale.Position        = [0.108,postStart-0.36,0.05,0.05];
            obj.edit_dispScale.FontSize        = 15;
            obj.edit_dispScale.String          = num2str(1);
            
            % object with properties of UIControl checkbox_uvwPlot
            obj.checkbox_uvwPlot                    = uicontrol('Parent',obj.fig,'Style','checkbox');
            obj.checkbox_uvwPlot.Units              = 'normalized';
            obj.checkbox_uvwPlot.Position           = [0.008,postStart-0.4,0.15,0.03];
            obj.checkbox_uvwPlot.String             = 'display result';
            obj.checkbox_uvwPlot.FontSize           = 15;
            obj.checkbox_uvwPlot.Value              = 1; 
            
            % object with properties of UIControl checkbox_nodesPlot
            obj.checkbox_nodesPlot                  = uicontrol('Parent',obj.fig,'Style','checkbox');
            obj.checkbox_nodesPlot.Units            = 'normalized';
            obj.checkbox_nodesPlot.Position         = [0.008,postStart-0.43,0.15,0.03];
            obj.checkbox_nodesPlot.String           = 'display nodes';
            obj.checkbox_nodesPlot.FontSize         = 15;
            obj.checkbox_nodesPlot.Value            = 1;
            
            % object with properties of UIControl checkbox_nodesText
            obj.checkbox_nodesText                  = uicontrol('Parent',obj.fig,'Style','checkbox');
            obj.checkbox_nodesText.Units            = 'normalized';
            obj.checkbox_nodesText.Position         = [0.008,postStart-0.46,0.15,0.03];
            obj.checkbox_nodesText.String           = 'nodes text';
            obj.checkbox_nodesText.FontSize         = 15;
            obj.checkbox_nodesText.Value            = 1;
            
            % object with properties of UIControl checkbox_eleText
            obj.checkbox_eleText                    = uicontrol('Parent',obj.fig,'Style','checkbox');
            obj.checkbox_eleText.Units              = 'normalized';
            obj.checkbox_eleText.Position           = [0.008,postStart-0.49,0.15,0.03];
            obj.checkbox_eleText.String             = 'elements text';
            obj.checkbox_eleText.FontSize           = 15;
            obj.checkbox_eleText.Value              = 1; 
            
            % object with properties of UIControl radioGroup
            obj.radioGroup(1) = uicontrol(  'Style',            'radiobutton',...
                                            'Units',            'normalized',...
                                            'Position',         [0.008,postStart-0.54,0.15,0.03],...
                                            'String',           '3D View (X-Y-Z)',...
                                            'FontSize',         15,...
                                            'Value',            1);
                                        
            obj.radioGroup(2) = uicontrol(  'Style',            'radiobutton',...
                                            'Units',            'normalized',...
                                            'Position',         [0.008,postStart-0.57,0.15,0.03],...
                                            'String',           '2D View (X-Y)',...
                                            'FontSize',         15,...
                                            'Value',            0);
                                        
            obj.radioGroup(3) = uicontrol(  'Style',            'radiobutton',...
                                            'Units',            'normalized',...
                                            'Position',         [0.008,postStart-0.6,0.15,0.03],...
                                            'String',           '2D View (X-Z)',...
                                            'FontSize',         15,...
                                            'Value',            0); 
                                        
            obj.radioGroup(4) = uicontrol(  'Style',            'radiobutton',...
                                            'Units',            'normalized',...
                                            'Position',         [0.008,postStart-0.63,0.15,0.03],...
                                            'String',           '2D View (Y-Z)',...
                                            'FontSize',         15,...
                                            'Value',            0); 
        end
        function longShortAx(obj)
            
            for i=1:length(obj.vecObj_nodes)
                
                jointsXcoord(i) = obj.vecObj_nodes(i).x;
                jointsZcoord(i) = obj.vecObj_nodes(i).z;
            end
            i = 0;
            
            obj.jointsXmin = min(jointsXcoord);
            obj.jointsXmax = max(jointsXcoord);
            
            obj.jointsZmin = min(jointsZcoord);
            obj.jointsZmax = max(jointsZcoord);
            
            if abs(obj.jointsXmax-obj.jointsXmin) >= abs(obj.jointsZmax-obj.jointsZmin)
                
                % delsi rozmer konstrukce je v X-ovem smeru (nebo stejne)
                obj.longerAxisLength = abs(obj.jointsXmax-obj.jointsXmin);
                
                if abs(obj.jointsZmax-obj.jointsZmin) <= obj.longerAxisLength/5
                    
                    obj.shorterAxisLength = obj.longerAxisLength/5;
                else
                    obj.shorterAxisLength = abs(obj.jointsZmax-obj.jointsZmin);
                end
            else
                % delsi rozmer konstrukce je v Z-ovem smeru
                obj.longerAxisLength = abs(obj.jointsZmax-obj.jointsZmin);
                
                if abs(obj.jointsXmax-obj.jointsXmin) <= obj.longerAxisLength/5
                    
                    obj.shorterAxisLength = obj.longerAxisLength/5;
                else
                    obj.shorterAxisLength = abs(obj.jointsXmax-obj.jointsXmin);
                end
            end
        end
    end
end

