% =================================================================== %
% %                                                                 % %
% %                                                                 % %
% %                        -----------------                        % %
% %                         C A L C U L U S                         % %
% %                        -----------------                        % %
% %                                                                 % %
% %                                                                 % %
% =================================================================== %
function [] = cllbck_button_launch(hObject, eventdata)

clc;
cla reset; % reset current plot

handles = guidata(hObject); % get the handles and user-defined data 
%% DESK PREPARATION

% JUST FOR HELP
n                       = 9;            % number of generalized coordinates of one node
hlp                     = 0;            % variable for help
onOffwarnConn           = 0;
onOffwarnDisp           = 0;
qMax                    = 0;
P                       = diag([1 -1 1 -1]);
z44                     = zeros(4,4);
z22                     = zeros(2,2);
z42                     = zeros(4,2);
z24                     = z42.';
theShortestConnection   = [];

% DECLARATION FOR FEM
CEL     = [];                   % lokalization table
KK      = [];                   % total stiffness matrix
f       = [];                   % vector of total forces
Lambda  = diag(ones(n,1));      % submatrix of transformation matrix T
J       = zeros(2*n);           % permutation matrix
%% DATA-LOADING 

folderID        = handles.edit_inputFolder.String;
userElementSize = str2num(handles.edit_elSize.String);

try           
    
[vecObj_nodes       ,...
 vecObj_forces      ,...
 vecObj_BCs         ,...
 vecObj_connections ,...
 qStart             ] = fce_dataLoading(folderID,n);

handles.vecObj_forces   = vecObj_forces;
handles.vecObj_BCs      = vecObj_BCs;
handles.qStart          = qStart;
catch    
    display('=========================================================================================================')
    warning([   'UNIframe is trying to say that SOMETHING IS WRONG IN SECTION -DATA LOADING-'                                               char(10),...
                                                                                                                                            char(10),...
                '1. CHECK IF YOU...TYPED NAME OF LOADING FOLDER CORRECTLY (YOU TYPED THIS NAME: ',folderID,' )'                             char(10),...
                '2. CHECK IF YOU...HAVE IN LOADING FOLDER 4 FILES WHICH CONTAINS INPUT DATA: JOINTS, CONNECTIONS, FORCES and BOUNDARIES'    char(10),...
                '3. CHECK IF YOU...NAMED 4 FILES WITH INPUT DATA SAME LIKE YOUR LOADING FOLDER (files extensions are different)'            char(10),...                
                '4. CHECK IF YOU...DID NOT FORGET INSERT SOME INPUT DATA TO LOADING FILES (e.g. define magnitude of force)'                 char(10),...
                '5. CHECK IF YOU...INSERT YOUR INPUT DATA WITHOUT ANY MISTAKE (e.g. using number of joint that have not been defined)'      char(10),...
                '========================================================================================================='                 char(10),...
                'Warning text generated by Matlab:'                                                                                         char(10),...
                ]);
    display('=========================================================================================================')
    return
end
handles.n = n;
%% CONNECTIONS

for i=1:length(vecObj_connections)
    
    vecObj_connections(i).number = i;
    
    jointAcoord   = [vecObj_connections(i).pointA.x; 0 ;vecObj_connections(i).pointA.z];
    jointBcoord   = [vecObj_connections(i).pointB.x; 0 ;vecObj_connections(i).pointB.z];
    
    ABvector            = jointBcoord-jointAcoord;
    ABdistance          = sqrt(ABvector'*ABvector);
    
    if isempty(theShortestConnection) == 1
        
        theShortestConnection = ABdistance;
    else
        if ABdistance < theShortestConnection
            
            theShortestConnection = ABdistance;
        else
            % nothing to do
        end
    end
    
    handles.elementAdvice.String = ['==( minLength: ',num2str(theShortestConnection),' )=='];
    
    try
        roundDown           = floor(ABdistance/userElementSize)             ;
        roundUp             = ceil(ABdistance/userElementSize)              ;
        roundDownDifference = abs(userElementSize-(ABdistance/roundDown))   ;
        roundUpDifference   = abs(userElementSize-(ABdistance/roundUp))     ;
        
        if roundDownDifference < roundUpDifference
            
            accurateDeviation = roundDown;  % accurateDeviation is also quantity of elements in actual connection
        else
            accurateDeviation = roundUp;
        end

        for j=1:(accurateDeviation)-1
            
            vecObj_nodes(end+1)         = class_node;
            vecObj_nodes(end).number    = length(vecObj_nodes);
            vecObj_nodes(end).x         = jointAcoord(1)+(j*(ABdistance/accurateDeviation)*(ABvector(1)/ABdistance));
            vecObj_nodes(end).z         = jointAcoord(3)+(j*(ABdistance/accurateDeviation)*(ABvector(3)/ABdistance));
            vecObj_nodes(end).q         = reshape(qStart+1:qStart+n,1,n);
            
            qStart                          = vecObj_nodes(end).number*n;
            vecObj_connections(i).nodes(j)  = vecObj_nodes(end);
        end
        j = 0;

%         lel     = ABdistance/accurateDeviation;
%         KeTilda = fce_stiffMatrixLC(vecObj_connections(i),lel,P,J,z22,z44,z24,z42);
%         lam     = vecObj_connections(i).lambda;
%         
%         T([1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18],...
%           [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18]) = blkdiag(lam,lam);                                   
% 
%         Ke = T.'*KeTilda*T;
        
        lel     = ABdistance/accurateDeviation;
        lam     = vecObj_connections(i).lambda;
        T([1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18],...
          [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18]) = blkdiag(lam,lam);

        for j=1:accurateDeviation
            
            vecObj_elements(hlp+j)      = class_element;
            vecObj_elements(end).number = j+hlp;
            
            if j == 1 && accurateDeviation == 1
                
                vecObj_elements(end).nodeA = vecObj_connections(i).pointA;
                vecObj_elements(end).nodeB = vecObj_connections(i).pointB;
                
                vecObj_connections(i).elements(j) = vecObj_elements(end);    
       
            elseif j == 1
                
                vecObj_elements(end).nodeA = vecObj_connections(i).pointA;
                vecObj_elements(end).nodeB = vecObj_connections(i).nodes(1);
                
                vecObj_connections(i).elements(j) = vecObj_elements(end);
   
            elseif j == accurateDeviation
                
                vecObj_elements(end).nodeA = vecObj_connections(i).nodes(end);
                vecObj_elements(end).nodeB = vecObj_connections(i).pointB;
                
                vecObj_connections(i).elements(j) = vecObj_elements(end);
            else
                vecObj_elements(end).nodeA = vecObj_connections(i).nodes(j-1);
                vecObj_elements(end).nodeB = vecObj_connections(i).nodes(j);
                
                vecObj_connections(i).elements(j) = vecObj_elements(end);
            end
            
            CELi(j,:) = [vecObj_connections(i).elements(j).nodeA.q vecObj_connections(i).elements(j).nodeB.q];
            % CELi(j,[4 7 13 16]) = 0;
            
            %%%%%% added for Compo %%%%%%
            KeTilda = fce_stiffMatrixLC(vecObj_connections(i),lel,P,J,z22,z44,z24,z42,j,accurateDeviation);
            Ke = T.'*KeTilda*T;
            
            if max(CELi(j,:)) > qMax
                                
                qMax = max(CELi(j,:));
                
                f(end+1:qMax)               = 0;
                KK(end+1:qMax,end+1:qMax)   = 0;
            end
            
            for k=1:length(vecObj_BCs)
                
                if (vecObj_connections(i).pointA.number == vecObj_BCs(k).node.number) && j == 1
                    
                    CELi(j,1:9) = CELi(1,1:9).*vecObj_BCs(k).permit.';
                    
                elseif (vecObj_connections(i).pointB.number == vecObj_BCs(k).node.number) && j == accurateDeviation
                    
                    CELi(j,10:18) = CELi(end,10:18).*vecObj_BCs(k).permit.';
                else
                    % nothing to do
                end
            end
            k = 0;
            
            kod             = CELi(j,:);
            nonZeroIndex    = find(kod);
            kodKK           = kod(nonZeroIndex);
            
            KK(kodKK,kodKK) = KK(kodKK,kodKK) + Ke(nonZeroIndex,nonZeroIndex);
        end
        hlp = hlp+j;
        j   = 0;
        
        handles.vecObj_elements = vecObj_elements;

%         for j=1:length(vecObj_BCs)
%             
%             if vecObj_connections(i).pointA.number == vecObj_BCs(j).node.number
%                 
%                 CELi(1,1:9) = CELi(1,1:9).*vecObj_BCs(j).permit.';
%                 
%             elseif vecObj_connections(i).pointB.number == vecObj_BCs(j).node.number
%                 
%                 CELi(end,10:18) = CELi(end,10:18).*vecObj_BCs(j).permit.';
%             else
%                 % nothing to do
%             end
%         end
%         j = 0;

        CEL  = [CEL; CELi];
        CELi = [];
    catch
        if onOffwarnConn == 0

            display('=========================================================================================================')
            warning([   'UNIframe is trying to say that SOMETHING IS WRONG IN SECTION -CONNECTIONS-'                                char(10),...
                                                                                                                                    char(10),...
                '1. CHECK IF YOU...INSERT VALUE FOR ELEMENT SIZE (your system might not be discretized)'                            char(10),...
                '2. CHECK IF YOU...HAVE NUMBER VALUE FOR ELEMENT SIZE (combination of numbers with letter is not permitted)'        char(10),...                
                '========================================================================================================='         char(10),...
                'Warning text generated by Matlab:'                                                                                 char(10),...
                ]);
            display('=========================================================================================================')
            onOffwarnConn = 1;
        else
            % nothing to do
        end
    end
    handles.onOffwarnConn = onOffwarnConn;
end
hlp = 0;
i   = 0;

handles.vecObj_nodes        = vecObj_nodes;
handles.vecObj_connections  = vecObj_connections;
%% FORCES

for i=1:length(vecObj_forces)
    
    if vecObj_forces(i).angle(1) == -1
        
        if vecObj_forces(i).angle(2) == -1
            
            % activation the bending moment in construction plane XZ
            f(vecObj_forces(i).node.q(6)) = vecObj_forces(i).intensite;
        else
            for j=1:length(vecObj_connections)
                
                if vecObj_forces(i).node.number == vecObj_connections(j).pointA.number

                    f(vecObj_forces(i).node.q(3)) = vecObj_forces(i).intensite*sin(vecObj_forces(i).angle(2)*(pi/180) - vecObj_connections(j).angle)*sin(pi/2 - vecObj_connections(j).angle) +... 
                                                    vecObj_forces(i).intensite*cos(vecObj_forces(i).angle(2)*(pi/180) - vecObj_connections(j).angle)*sin(vecObj_connections(j).angle);
                    
                    f(vecObj_forces(i).node.q(8)) = vecObj_forces(i).intensite*sin(vecObj_forces(i).angle(2)*(pi/180) - vecObj_connections(j).angle)*(-cos(pi/2 - vecObj_connections(j).angle)) +... 
                                                    vecObj_forces(i).intensite*cos(vecObj_forces(i).angle(2)*(pi/180) - vecObj_connections(j).angle)*cos(vecObj_connections(j).angle);                                    
                    
                elseif vecObj_forces(i).node.number == vecObj_connections(j).pointB.number  

                    f(vecObj_forces(i).node.q(3)) = vecObj_forces(i).intensite*sin(vecObj_forces(i).angle(2)*(pi/180) - vecObj_connections(j).angle)*sin(pi/2 - vecObj_connections(j).angle) +... 
                                                    vecObj_forces(i).intensite*cos(vecObj_forces(i).angle(2)*(pi/180) - vecObj_connections(j).angle)*sin(vecObj_connections(j).angle);
                    
                    f(vecObj_forces(i).node.q(8)) = vecObj_forces(i).intensite*sin(vecObj_forces(i).angle(2)*(pi/180) - vecObj_connections(j).angle)*(-cos(pi/2 - vecObj_connections(j).angle)) +... 
                                                    vecObj_forces(i).intensite*cos(vecObj_forces(i).angle(2)*(pi/180) - vecObj_connections(j).angle)*cos(vecObj_connections(j).angle); 
                else
                    % nothing to do
                end
            end
            j = 0;
        end 
    else 
        f(vecObj_forces(i).node.q(1)) = vecObj_forces(i).intensite*cos(vecObj_forces(i).angle(1)*pi/180)*cos(vecObj_forces(i).angle(2)*pi/180);
        f(vecObj_forces(i).node.q(2)) = vecObj_forces(i).intensite*cos(vecObj_forces(i).angle(1)*pi/180)*sin(vecObj_forces(i).angle(2)*pi/180);
        f(vecObj_forces(i).node.q(5)) = vecObj_forces(i).intensite*sin(vecObj_forces(i).angle(1)*pi/180);
    end
end
i = 0;
%% DISPLACEMENTS

try
    qNum = [1:1:size(KK,1)]'; % sequence of generalized coordinates numbers of total system
    
    rubberIndex = 1;
    for i=1:size(KK,1)
        
        if KK(i,i) == 0
            
            rubber(rubberIndex) = i;
            rubberIndex = rubberIndex+1;
        else
            %do nothing
        end
    end
    i = 0;
    
    KK(:,rubber)    = [];
    KK(rubber,:)    = [];
    f(rubber)       = [];
    qNum(rubber)    = [];
    
    CEL
    
    q = inv(KK)*f.';
    
    for i=1:length(vecObj_nodes)
        
        disp(['Uzel c.: ',num2str(vecObj_nodes(i).number)])
        disp(['Zobecnene souradnice: ',num2str(vecObj_nodes(i).q)])
        fprintf('\n') % blank line
    end
    i = 0;
    
    RSLTs = [qNum q] % results 
    handles.RSLTs = RSLTs;
catch
    if onOffwarnDisp == 0
        
        display('=========================================================================================================')
        warning([   'UNIframe is trying to say that SOMETHING IS WRONG IN SECTION -DISPLACEMENTS-'                              char(10),...
                                                                                                                                char(10),...
            '1. CHECK IF YOU...SEE ANNOUNCEMENT ABOUT SECTION -CONNECTIONS-, THAN PROBLEM IS NOT IN SECTION -DISPLACEMENTS-'    char(10),...
            '2. CHECK IF YOU...KNOW THAT PREVIOUS POINT IS NOT TRUE, YOU SHOULD INSPECT SECTION -DISPLACEMENT-'                 char(10),...
            '========================================================================================================='         char(10),...
            'Warning text generated by Matlab:'                                                                                 char(10),...
            ]);
        display('=========================================================================================================')
        onOffwarnDisp = 1;
    else
        % nothing to do
    end 
end
handles.onOffwarnDisp = onOffwarnDisp; 
%% VISUALIZATION

handles.longShortAx();

fce_Ploting(handles);

% handles.Ax.Color = GUIbckgrnd;
% 
% handles.Ax.XLabel.String    = 'X';
% handles.Ax.YLabel.String    = 'Y';
% handles.Ax.ZLabel.String    = 'Z';
%% GUI-UPDATE

guidata(hObject, handles); % need to update my GUI structure (guiStruc) - maybe not necessary
%% NOTES

% disp('dilci datove soubory ve slozce s danym prikladem musi mit pred teckou totozny nazev jako ta slozka')
% disp('cisla stycniku musi byt posloupnost prirozenych cisel postupne po sobe jdoucich od nejmensiho po nejvetsi')
% disp('vsechny stycniky musi byt nejakym zpusobem propojeny')
% disp('nelze udelat spojeni z urciteho stycniku do sebe sameho')
% disp('zalezi na uhlu natoceni prvku, resp je treba treba rozeznavat 0 a 180 stupnu')
% disp('smer cesty vypoctoveho schematu dane konfigurace musi jit VZDY ve stejnem smyslu (zleva do prava)')
% disp('kdyz se pri definici zatizeni zada pro uhly specialni pripad [-1;-1], tak to znaci zatizeni momentem v rovine XZ')
% disp('kdyz se pri definici zatizeni zada pro uhly specialni pripad [-1;<0,360>], tak to znaci zatizeni momentem jehoz vektor se nataci v rovine XZ v intervalu <0,360> stupnu')
% disp('znamenko u zatizeni jak silou, tak momenty, muze byt ruzne')
end